'use strict';
var util = require('util');
var path = require('path');
var yeoman = require('yeoman-generator');
var yosay = require('yosay');
var chalk = require('chalk');
var _s = require('underscore.string');

var AccessDrupalThemeGenerator = yeoman.generators.Base.extend({
  initializing: function () {
    this.pkg = require('../package.json');
  },

  prompting: function () {
    var done = this.async();

    // Have Yeoman greet the user.
    this.log(yosay(
      'Welcome to the priceless Access Drupal Theme generator!'
    ));

    var prompts = [{
      type: 'string',
      name: 'projectName',
      message: 'What\'s your theme\'s name?' + chalk.red(' (Required) ') + chalk.blue('Eg. Client Name Theme'),
      validate: function (input) {
        if (input === '') {
          return 'Please enter your theme\'s name';
        }
        return true;
      }
    }];

    this.prompt(prompts, function (props) {
      this.projectName = props.projectName;
      this.projectSlug = _s.underscored(props.projectName);

      done();
    }.bind(this));
  },

  writing: {
    drupal: function () {
      // Make Theme Folder
      this.dest.mkdir(this.projectSlug);
      this.destinationRoot(this.projectSlug);

      // Templates section
      this.dest.mkdir('templates');
      this.dest.mkdir('templates/block');
      this.dest.mkdir('templates/field');
      this.dest.mkdir('templates/html');
      this.dest.mkdir('templates/maintenance');
      this.dest.mkdir('templates/node');
      this.dest.mkdir('templates/page');
      this.dest.mkdir('templates/region');
      this.dest.mkdir('templates/search');

      // Template block files
      this.template('drupal/templates/block/_block--no-wrapper.tpl.php', 'templates/block/block--no-wrapper.tpl.php');
      this.template('drupal/templates/block/_block.tpl.php', 'templates/block/block.tpl.php');

      // Template field files
      this.template('drupal/templates/field/_field.tpl.php', 'templates/field/field.tpl.php');

      // Template html files
      this.template('drupal/templates/html/_html.tpl.php', 'templates/html/html.tpl.php');

      // Template maintenance files
      this.template('drupal/templates/maintenance/_maintenance-page.tpl.php', 'templates/maintenance/maintenance-page.tpl.php');

      // Template node files
      this.template('drupal/templates/node/_node--mode-full.tpl.php', 'templates/node/node--mode-full.tpl.php');
      this.template('drupal/templates/node/_node--mode-teaser.tpl.php', 'templates/node/node--mode-teaser.tpl.php');
      this.template('drupal/templates/node/_node.tpl.php', 'templates/node/node.tpl.php');

      // Template page files
      this.template('drupal/templates/page/_page.tpl.php', 'templates/page/page.tpl.php');

      // Template region files
      this.template('drupal/templates/region/_region--no-wrapper.tpl.php', 'templates/region/region--no-wrapper.tpl.php');
      this.template('drupal/templates/region/_region.tpl.php', 'templates/region/region.tpl.php');

      // Template search files
      this.template('drupal/templates/search/_search-block-form.tpl.php', 'templates/search/search-block-form.tpl.php');
      this.template('drupal/templates/search/_search-result.tpl.php', 'templates/search/search-result.tpl.php');
      this.template('drupal/templates/search/_search-results.tpl.php', 'templates/search/search-results.tpl.php');


      // Includes section
      this.dest.mkdir('includes');

      // Includes files
      this.template('drupal/includes/_form.inc', 'includes/form.inc');
      this.template('drupal/includes/_menu.inc', 'includes/menu.inc');
      this.template('drupal/includes/_theme.inc', 'includes/theme.inc');
      this.template('drupal/includes/_views.inc', 'includes/views.inc');


      // Template file
      this.template('drupal/_template.php', 'template.php');


      // Info file
      this.template('drupal/_theme.info', this.projectSlug + '.info');
    },

    styles: function () {
      // Create partial folders
      this.dest.mkdir('styles');
      this.dest.mkdir('styles/base');
      this.dest.mkdir('styles/layout');
      this.dest.mkdir('styles/module');
      this.dest.mkdir('styles/settings');
      this.dest.mkdir('styles/state');
      this.dest.mkdir('styles/theme');
      this.dest.mkdir('styles/tools');

      // Add gitkeep's to each folder in case empty
      this.src.copy('gitkeep', 'styles/base/.gitkeep');
      this.src.copy('gitkeep', 'styles/layout/.gitkeep');
      this.src.copy('gitkeep', 'styles/module/.gitkeep');
      this.src.copy('gitkeep', 'styles/settings/.gitkeep');
      this.src.copy('gitkeep', 'styles/state/.gitkeep');
      this.src.copy('gitkeep', 'styles/theme/.gitkeep');
      this.src.copy('gitkeep', 'styles/tools/.gitkeep');

      // Copy base partials
      this.src.copy('styles/base/_elements.scss', 'styles/base/_elements.scss');
      this.src.copy('styles/base/_reset.scss', 'styles/base/_reset.scss');

      // Copy layout partials
      this.src.copy('styles/layout/_grid.scss', 'styles/layout/_grid.scss');

      // Copy settings partials
      this.src.copy('styles/settings/_config.scss', 'styles/settings/_config.scss');
      this.src.copy('styles/settings/_variables.scss', 'styles/settings/_variables.scss');

      // Copy tools partials
      this.src.copy('styles/tools/_mixins.scss', 'styles/tools/_mixins.scss');

      // Copy main sass files
      this.src.copy('styles/global.scss', 'styles/global.scss');
    },

    scripts: function () {
      // Create javascript folder
      this.dest.mkdir('js');

      // Copy the global script file
      this.src.copy('scripts/global.js', 'js/global.js');
    },

    assets: function () {
      // Create assets folder
      this.dest.mkdir('images');
      this.dest.mkdir('images/favicons');
      this.dest.mkdir('fonts');

      // Add gitkeep's to each folder in case empty
      this.src.copy('gitkeep', 'fonts/.gitkeep');

      // Copy the favicon template
      this.src.copy('images/favicons/favicon.png', 'images/favicons/favicon.png');

      // Copy Drupal theme logo
      this.src.copy('images/logo.png', 'logo.png');

      // Copy Drupal screenshot
      this.src.copy('images/screenshot.png', 'screenshot.png');
    },

    projectfiles: function () {
      this.src.copy('_README.md', 'README.md');

      this.template('_bower.json', 'bower.json');
      this.template('_Gruntfile.js', 'Gruntfile.js');
      this.template('_package.json', 'package.json');

      this.src.copy('editorconfig', '.editorconfig');
      this.src.copy('gitattributes', '.gitattributes');
      this.src.copy('gitignore', '.gitignore');
      this.src.copy('jshintrc', '.jshintrc');
    }
  },

  end: function () {
    this.installDependencies();
  }
});

module.exports = AccessDrupalThemeGenerator;
